/*FILE: bisection.c

Programer: Sunethra Ramanan suna@physics.iitm.ac.in

Date: 13 Feb 2012

Version: Original

Revision-History:

Comments:
13 Feb 2012: This code finds the solution s of a given function such that f(s) = 0 to within a tolerance using the bisection method.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/******************** Function Prototype **************************************/
double 
bisection(int max_iter, double eps, double delta, double a, double b, double (*func)(double, void *), void *params_ptr);
/******************************************************************************/

double 
bisection(int max_iter, double eps, double delta, double a, double b, double (*func)(double, void *), void *params_ptr)
{
	int n;								/*no. of iteration*/
	
	double mid;						/*midpoint*/
	double fmid;					/*value of the function at the midpoint*/
	double fa, fb;				/*value of the function at a and b*/
	
	/*intialize*/
	n = 0;
	
	fa = (* func)(a, params_ptr);
	fb = (* func)(b, params_ptr);
	
	if(fa * fb > 0) /*this ensures that a and b bracket the solution*/
	{
		printf("there is no root in the given interval [%f:%f] \n", a,b);
		
		exit(1);
	}
	
	else
	{
		if( n <= max_iter)
		{
			do
			{
				/*initializing mid*/
				mid = (a+b)/2.;
			
				/*initializing fmid*/
				fmid = (* func)(mid, params_ptr);
			
				if(fmid * fa > 0.)
				{
					a = mid;
					
					fa = fmid;
					/*printf("mid:%e fmid:%e \n", mid, fmid);*/
				}
				else
				{
					b = mid;
					
					fb = fmid;
					
					/*printf("mid:%e fmid:%e \n", mid, fmid);*/
				}
				
				/*increment n*/
				n ++;
			}
			while(fabs(b - a) > eps || fabs(fmid) > delta);
		}
		else
		{
			printf("too many iterations: %d! \n", n);
			exit(1);
		}
		
		printf("no. of iterations:%d \n", n);
	}
	
	return(mid);
}

/***************************************************************************/
/****************************************************************************/